using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;

namespace Cotyounoyume{
	public class CollisionFailureDetector
	{
		public EBATCore core;
		// public float failureCheckThresholdTime = 7.0f;
		public Dictionary<string, CollisionTrigger> triggerMap;
		public Dictionary<string, bool> checkOnMap;
		// public Dictionary<string, bool> failureMap;
		// public Dictionary<string, bool> alwaysOnMap;
		public Dictionary<string, float> lastDeactiveTimeMap;
		public Dictionary<string, float> lastActiveTimeMap;
		public List<string> cTriggerNames = new List<string>(){"MouthTrigger", "ThroatTrigger","rNippleTrigger", "lNippleTrigger", "LabiaTrigger", "VaginaTrigger", "DeepVaginaTrigger", "DeeperVaginaTrigger"};
		
		public float alwaysOnThreshold = 1f;
		public float alwaysFailureThreshold = 1f;
		public float[] cuaStageThreshold = {
			// 3.2f, 3.0f, 2.8f, 2.5f,
			1.2f, 1.0f, 0.8f, 0.5f,
		};
		
		public CollisionFailureDetector(EBATCore core){
			this.core = core;
			triggerMap = new Dictionary<string, CollisionTrigger>();
			checkOnMap = new Dictionary<string, bool>();
			// failureMap = new Dictionary<string, bool>();
			// alwaysOnMap = new Dictionary<string, bool>();
			lastActiveTimeMap = new Dictionary<string, float>();
			lastDeactiveTimeMap = new Dictionary<string, float>();
			
			Init();
		}
		
		public void Init(){
			OnDestroy();
			
			foreach(string cTriggerName in cTriggerNames){
				triggerMap.Add(cTriggerName, core.atom.GetStorableByID(cTriggerName) as CollisionTrigger);
				checkOnMap.Add(cTriggerName, false);
				// failureMap.Add(cTriggerName, false);
				// alwaysOnMap.Add(cTriggerName, false);
				lastActiveTimeMap.Add(cTriggerName, 0f);
				lastDeactiveTimeMap.Add(cTriggerName, 0f);
			}
		}
		
		public void OnDestroy(){
			triggerMap.Clear();
			checkOnMap.Clear();
			// failureMap.Clear();
			lastActiveTimeMap.Clear();
		}

		public bool GetTriggerStatus(string name){
			return GetTrigger(name).trigger.active;
		}

		public CollisionTrigger GetTrigger(string name){
			if(!triggerMap.ContainsKey(name)){
				return null;
			}
			return triggerMap[name];
		}
		
		public void ActivateTrigger(string name, bool isOn = true){
			var status = GetTriggerStatus(name);
			// Utility.Debug($"routeX: name:{name}, isOn:{isOn}, status:{status}");
			if(status == isOn) return;
			try{
				if(!triggerMap.ContainsKey(name)) return;
				core.StartCoroutine(ActivateTriggerCoroutine(triggerMap[name], isOn));

			}catch(Exception e){
				Utility.LoggerE(e);
			}
		}
		
		public IEnumerator ActivateTriggerCoroutine(CollisionTrigger cTrigger, bool isOn = true){
			// CollisionTrigger cTrigger = triggerMap[name];
			
			cTrigger.trigger.active = !isOn;
			yield return new WaitForSeconds(0.1f);
			cTrigger.trigger.active = isOn;
			// yield return null;
		}
		
		public void Subscribe(string name){
			if(name.EndsWith("NippleControl")){
				name = name.Replace("Control", "Trigger");
			}
			if(!triggerMap.ContainsKey(name)){
				core.Info("ERROR: unknown trigger: " + name);
				return;
			}

			checkOnMap[name] = true;
			lastActiveTimeMap[name] = core.currentTime + 5f;
			lastDeactiveTimeMap[name] = core.currentTime + 5f;
		}
		
		public void Unsubscribe(string name){
			if(name.EndsWith("NippleControl")){
				name = name.Replace("Control", "Trigger");
			}
			if(!triggerMap.ContainsKey(name)){
				return;
			}
			
			checkOnMap[name] = false;
			lastActiveTimeMap[name] = 0f;
			lastDeactiveTimeMap[name] = 0f;
		}
				
		public void Update(){
			if(!core.collisionFailureCheckOnGlobal) return;
			// debugPrint();
			CheckCollisions();
			// RingFailureCollisions();
		}

		public void UpdateForTouch(){

		}
		
		public void CheckCollisions(){
			cTriggerNames.ForEach(name => CheckCollision(name));
		}

		public void ResetPhysics(){
			if(!core.collisionAutoFixOnGlobal) return;
			core.atom.ResetPhysics(false, false);
		}

		public void CheckCollision(string name){
			if(!triggerMap.ContainsKey(name)) return;
			if(triggerMap[name].trigger.active){
				lastActiveTimeMap[name] = core.currentTime;
			}else{
				lastDeactiveTimeMap[name] = core.currentTime;
			}

			if(core.currentTime > lastDeactiveTimeMap[name] + alwaysOnThreshold){
				ActivateTrigger(name, false);
			}
			if(!checkOnMap.ContainsKey(name)) return;
			if(!checkOnMap[name]) return; 

			var offset = cuaStageThreshold[core.GetCUAStage()];
			if(core.currentTime > lastActiveTimeMap[name] + offset){
				ActivateTrigger(name, true);
				ResetPhysics();
			}
		}

		public bool BreastAndPubic(){
			bool pubicOn  = checkOnMap["LabiaTrigger"] || checkOnMap["VaginaTrigger"];
			bool breastOn = checkOnMap["rNippleTrigger"] || checkOnMap["lNippleTrigger"];
		// public List<string> cTriggerNames = new List<string>(){"MouthTrigger", "ThroatTrigger","rNippleTrigger", "lNippleTrigger", "LabiaTrigger", "VaginaTrigger", "DeepVaginaTrigger", "DeeperVaginaTrigger"};

			return pubicOn && breastOn;
		}

	}
}
