#region using
using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.Threading;
using System.Text.RegularExpressions;
#endregion

namespace Cotyounoyume{
	public class Tears
	{
		private Atom _atom;
		public float TearsClosingTimeOffset { get; set; } = 10f;

		private EBATCore _core;
		public bool DisableLog { get; set; }
		public bool IsActive { get; set; } = false;

		public bool TearsOn = true;
		public bool DroolOn = true;
		public bool ForceActive { get; set; }
		public string TearsTypeDefault { get; set; } = "Clear";

		public ClothesManager _cm;

		public bool TearsGloss { get; set; } = true;
		public List<string> TearsTypes;

		private DAZCharacterSelector _geometryForClothes;
		private Dictionary<string, DAZClothingItem> _tearsMap;
		private string _currentColor = "Clear";
		private bool _isTearsAlt;
		private float _forceActiveTime = 0f;

		public Tears(EBATCore core)
		{
			try
			{
				_core = core;
				_atom = _core.atom;
				IsActive = false;
				_cm = new ClothesManager();
				_cm.Initialize(_atom);
			}
			catch (Exception e)
			{
				_core.LoggerE(e);
			}
		}

		public void LoadSettings()
		{
			try
			{
				TearsTypeDefault = _core.settings["General"]["Tears Type"];
				SetTearsType(TearsTypeDefault);
				TearsOn = _core.settings["General"]["Tears On"].AsBool;
				DroolOn = _core.settings["General"]["Drool On"].AsBool;
				TearsGloss = _core.settings["General"]["Tears Gloss On"].AsBool;
				TearsClosingTimeOffset = _core.settings["General"]["Tears Duration Time"].AsFloat;

				JSONNode jsThreshold = _core.settings["StageThreshold"];
				for (int i = 0; i < jsThreshold.Count; i++)
				{
					int stage = i + 1;
					if (jsThreshold[i]["Clothes"].Count > 0)
					{
						List<string> uids = new List<string>();
						for (int j = 0; j < jsThreshold[i]["Clothes"].Count; j++)
						{
							uids.Add(jsThreshold[i]["Clothes"][j]);
						}
						_cm.SetClothesByUids(stage, uids);
					}
				}

				TearsTypes = new List<string>();
				JSONNode jsTears = _core.settings["Tears"];
				for (int i = 0; i < jsTears.Count; i++)
				{
					TearsTypes.Add(jsTears[i]);
				}

			}
			catch (Exception e)
			{
				_core.LoggerE(e);
			}
		}

		public void ApplySettings() {
			JSONStorable geometryStorable = _atom.GetStorableByID("geometry");
			_geometryForClothes = geometryStorable as DAZCharacterSelector;

			_tearsMap = new Dictionary<string, DAZClothingItem>();
			List<DAZClothingItem> clothes = Utility.GetClothes(_atom);
			for (int i = 0; i < clothes.Count; i++) {
				if (clothes[i].creatorName != "cotyounoyume") { continue; }
				if (!clothes[i].uid.Contains("/Tears/") && !clothes[i].uid.Contains("/drool/")) { continue; }
				if (!clothes[i].isLatestVersion) { continue; }

				if (!_tearsMap.ContainsKey(clothes[i].displayName)) {
					_tearsMap.Add(clothes[i].displayName, clothes[i]);
				}
				else
				{
					if (clothes[i].version == "") {
						_tearsMap[clothes[i].displayName] = clothes[i];
					}
				}
			}
		}

		public void ForceActivateTemporary(float duration)
		{
			_core.StartCoroutine(ForceActivateTemporaryCoroutine(duration));
		}

		public IEnumerator ForceActivateTemporaryCoroutine(float duration)
		{
			ForceActive = true;
			ChangeTearsEquipment();
			_forceActiveTime = _core.currentTime + duration - 0.1f;
			yield return new WaitForSeconds(duration);
			ForceActive = _core.currentTime <= _forceActiveTime;
			ChangeTearsEquipment();
		}


		public void SetTearsType(string type)
		{
			_isTearsAlt = type.EndsWith("(Alt)");
			if (type.StartsWith("Normal"))
			{
				_currentColor = "Normal";
			}
			else
			{
				_currentColor = type.StartsWith("White") ? "White" : "Clear";
			}
		}

		public void Maintenance() { }

		public void ChangeTearsEquipment()
		{
			if (TearsOn == false)
			{
				return;
			}

			UnequipTearsAll();
			foreach (var item in GetActiveTearsList())
			{
				_geometryForClothes.SetActiveClothingItem(_tearsMap[item], true);
			}
		}

		public List<string> GetActiveTearsList()
		{
			var list = new List<string>();
			string tears = GetTearsName();
			if (tears != "") { list.Add(tears); }
			string drool = GetDroolName();
			if (drool != "") { list.Add(drool); }
			string snot = GetSnotName();
			if (snot != "") { list.Add(snot); }
			return list;
		}

		public bool IsInMultipleOrgasm()
		{
			if (ForceActive) { return true; }

			if (_core.GetStage() < 5) { return false; }
			if (!_core.customUi.jsHasMultipleOrgasms.val) { return false; }

			return true;
		}


		public string GetTearsName()
		{
			int stage = ForceActive ? 5 : _core.GetStage();
			if (stage == 0) { return ""; }

			string[] tearsArray = { "", "Small", "Middle", "Large", "Large", "Large" };
			string tearsSize = tearsArray[stage];
			string type = (_isTearsAlt) ? "(type2)" : "(type1)";

			if (IsInMultipleOrgasm())
			{
				tearsSize = "Large2";
				type = "(type1)";
			}
			return "Tears" + " " + tearsSize + " " + _currentColor + " " + type;
		}

		public string GetDroolName()
		{
			int stage = ForceActive ? 5 : _core.GetStage();
			if (stage < 4) { return ""; }

			return (IsInMultipleOrgasm()) ? ("Drool2 " + _currentColor + " (type1)") : ("Drool Normal " + _currentColor + " (type1)");
		}

		public string GetSnotName()
		{
			int stage = ForceActive ? 5 : _core.GetStage();
			if (!IsInMultipleOrgasm()) { return ""; }

			return "Snot " + _currentColor + " (type1)";
		}


		public void UnequipClothes(int stage)
		{
			if(_cm == null)	return;
			_cm.UnequipClothes(stage);
		}

		public void UndressAll()
		{
			if(_cm == null)	return;
			_cm.UndressAll();
		}

		public void SetClothes(int stage) 
		{
			if(_cm == null)	return;
			_cm.SetClothes(stage);
		}

		public List<string> GetClothUids(int stage)
		{
			if(_cm == null)	return new List<string>();
			return _cm.GetClothUids(stage);
		}

		public void ClearClothes(int stage)
		{
			if(_cm == null)	return;
			_cm.ClearClothes(stage);
		}

		public void ShowClothes(int stage)
		{
			if(_cm == null)	return;
			_cm.ShowClothes(stage);
		}

		public void EquipClothes(int stage)
		{
			if(_cm == null)	return;
			_cm.EquipClothes(stage);
		}
		
		public void UnequipTearsAll(){
			try{
				if(!_core.IsLoaded()){ return; }
				
				foreach(KeyValuePair<string, DAZClothingItem> cloth in _tearsMap){
					_geometryForClothes.SetActiveClothingItem(cloth.Value, active: false);					
				}
			}catch(Exception e){
				_core.LoggerE(e);
			}
		}
		
//		public void ChangeTearsEquipment(){
//			try{
////				unequipTears();
//				UnequipTearsAll();
//				_tearsString = "";
//				_droolString = "";
//				ChangeBlushEquipment();
//			}catch(Exception e){
//				_core.loggerE(e);
//			}
//		}

		public void Update() { }
		public void Manage() { }

		public void OnDestroy(){
			UnequipTearsAll();
			_cm.OnDestroy();
			_cm = null;
		}
	}
}